#include <glibmm/i18n.h>
#include <gtkmm.h>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <sstream>

#include <boost/format.hpp>
#include <mcs/mcs.h>

#include "file_utils.hpp"
#include "util.hpp"
#include "paths.hpp"
#include "main.hpp"
#include "uri++.hpp"

#include "library_ui_move_album.hpp"
#include "x_library.hpp"
#include "x_system_control.hpp"
#include "x_vfs.hpp"

namespace
{
  void
  cbe_set_sensitive (Gtk::CheckButton *cb, Gtk::ComboBoxEntry *cbe)
  {
      cbe->set_sensitive (cb->get_active());
  }

  void
  fcb_set_sensitive (Gtk::CheckButton *cb, Gtk::FileChooserButton *fcb)
  {
      fcb->set_sensitive (cb->get_active());
  }
}

namespace Bmp
{
      LibraryUIMoveAlbum::~LibraryUIMoveAlbum ()
      {
        mcs->key_set<std::string>("bmp","file-chooser-path-relocate-album", fcb_move->get_current_folder ());
      }

      LibraryUIMoveAlbum::LibraryUIMoveAlbum (BaseObjectType*                       cobject,
                                              const Glib::RefPtr<Gnome::Glade::Xml> &xml)
            : Gtk::Dialog (cobject),
              ref_xml     (xml),
              disc        (Gdk::Pixbuf::create_from_file (Glib::build_filename (DATA_DIR, "images" G_DIR_SEPARATOR_S "audio.png")))
      {
          ref_xml->get_widget ("coverart",
                                coverart);

          ref_xml->get_widget ("e_album",
                                e_album);

          ref_xml->get_widget ("e_artist",
                                e_artist);

          ref_xml->get_widget ("cb_rename",
                                cb_rename);

          ref_xml->get_widget ("cbe_rename",	
                                cbe_rename);

          ref_xml->get_widget ("fcb_move",
                                fcb_move);

          fcb_move->set_current_folder (mcs->key_get<std::string>("bmp","file-chooser-path-relocate-album"));

          ref_xml->get_widget ("cb_append_albumname",
                                cb_append_albumname);

          // FIXME: Implement: cb_rename->signal_toggled().connect (sigc::bind (sigc::ptr_fun (&cbe_set_sensitive), cb_rename, cbe_rename));
      }

      LibraryUIMoveAlbum*
      LibraryUIMoveAlbum::create ()
      {
        const std::string path = Glib::build_filename
              (DATA_DIR, "glade" G_DIR_SEPARATOR_S "library_ui_move_album.glade");
        Glib::RefPtr<Gnome::Glade::Xml> glade_xml = Gnome::Glade::Xml::create (path);
        LibraryUIMoveAlbum *library_ui = 0;
        glade_xml->get_widget_derived ("dialog_move_album", library_ui);
        return library_ui;
      }

      int
      LibraryUIMoveAlbum::run (const Glib::ustring& artist, const Glib::ustring& album) 
      {
          using namespace Glib;
          using namespace Gtk;
          using namespace Bmp::DB;
          using namespace Bmp::Library;
          using namespace std;

          AttributeList attrs; 
          string id_artist = library->get_metadatum_id (DATUM_ARTIST);
          string id_album  = library->get_metadatum_id (DATUM_ALBUM);
          attrs.push_back (Attribute (true, id_artist, artist));
          attrs.push_back (Attribute (true, id_album,  album));

          library->query (attrs, vector);
          if (vector.empty()) return GTK_RESPONSE_CANCEL;

          e_artist->set_text (artist);
          e_album->set_text (album);

          try {
            RefPtr<Gdk::Pixbuf> _cover;
            ustring asin = library->get_asin_for_album (album);
            Amazon::get_cover_by_asin (asin, _cover); 
            coverart->set (_cover);
          } catch (...) {
            coverart->set (disc);
          }
          return Gtk::Dialog::run ();
      }
}
